@@ -1,5 +1,5 @@
 /*
- * Copyright 2001 Compaq Information Technologies Group, L.P.
+ * Copyright 2001 Hewlett-Packard Development Company, L.P.
  * 
  * Compaq and the Compaq logo are trademarks of Compaq Information
  * Technologies Group, L.P. in the U.S. and/or other countries.
@@ -143,12 +143,6 @@ int _cmhp_ioctl(struct inode *i, struct file *f, unsigned int cmd,
       cmhpIoctlGetIMLRequest((PVOID)arg);
       return 0;
    }
-   else if(cmd == 0x34) //CMHP_SET_EVENT_NUMBER)
-   {
-//printk(KERN_ERR "cmhp ioctl cmd = %d\n", cmd);
-      cmhpIoctlSetEventNumber((PVOID)arg);
-      return 0;
-   }
    else
    {
       retval = cmhp_ioctl(cmd, arg);
