@@ -1,6 +1,6 @@
 /*++
 
-Copyright (c) 2000,2001 Compaq Computer Corporation
+(C) Copyright 2000-2005 Hewlett-Packard Development Company, L.P.
 
 Module Name:
 
@@ -11,11 +11,6 @@ Abstract:
     This module contains the code for a ring-0 Linux driver that
     provides kernel mode services to an application in user space.
 
-Author:
-
-    Tom.Bonola@compaq.com   Mar-2000
-    Chris.Frantz@compaq.com Mar-2001 Ported to Linux
-
 Environment:
 
     Kernel mode
@@ -59,7 +54,7 @@ void DbgDump(struct _CPQCIADAPTEROBJECT* Adapter, CPQCICONNECTION Connection);
 #define virt_to_page(v) MAP_NR(v)
 #endif
 
-#define CPQEFSVER	"7.2.0"
+#define CPQEFSVER	"7.4.0"
 
 #define STRINGIZE(x)	#x
 char kernel_version[] = "2.4.7-10smp";
@@ -153,46 +148,48 @@ void cpq_usleep(unsigned long usecs);
 //Creating, Deleting and Finding Connection Extensions
 //====================================================
 
+//A semaphore to guide the list of connections and
+//also make sure close_helper is called in a serial fashion.
+struct semaphore connection_lock; 
 
-struct semaphore connection_list_lock;
-
+//We can assume that the connection lock is engaged
 static CPQCICONNECTIONEXTENSION* cpqci_new_connection() {
 
-	CPQCICONNECTIONEXTENSION* c = kmalloc(sizeof(CPQCICONNECTIONEXTENSION), GFP_KERNEL);
+	CPQCICONNECTIONEXTENSION* c;
+
+	c = kmalloc(sizeof(CPQCICONNECTIONEXTENSION), GFP_KERNEL);
 	DBG("(CONNECTIONEXTENSION)kmalloc(%d) = %p\n", sizeof(CPQCICONNECTIONEXTENSION), c);
 	if (!c) return NULL;
 	
 	memset(c, 0, sizeof(CPQCICONNECTIONEXTENSION));
 	if (!CpqCi) return c;
 
-	down(&connection_list_lock);
 	c->Next = CpqCi->ConnectionList;
 	CpqCi->ConnectionList = c;
-	up(&connection_list_lock);
 	DBG("Insert CONNECTIONOBJECT into chain and returning %p\n", c);
 
 	return c;
 }
 
 
+//We can assume that the connection lock is engaged
 static CPQCICONNECTIONEXTENSION** cpqci_find_connection(CPQCICONNECTIONEXTENSION* c) {
 
 	CPQCICONNECTIONEXTENSION** pc;
 
 	if (!CpqCi) return NULL;
 
-	down(&connection_list_lock);
 	pc = &CpqCi->ConnectionList;
 	while (*pc) {
 		DBG("Matching Connection Object %p against %p\n", (*pc), c);
 		if ((*pc) == c) break; 
 		pc = &((*pc)->Next);
 	}
-	up(&connection_list_lock);
 
 	return pc;
 }
 
+//We can assume that the connection lock is engaged
 static void cpqci_delete_connection_mem_handle(CPQCICONNECTIONEXTENSION* c) {
 
         if (!c) return;	
@@ -217,6 +214,7 @@ static void cpqci_delete_connection_mem_handle(CPQCICONNECTIONEXTENSION* c) {
 	}
 }
 
+//We can assume that the connection lock is engaged
 static void cpqci_new_connection_mem_handle(CPQCICONTEXT Context, CPQCICONNECTIONEXTENSION* c, CPQCISIZE AllocationSize) {
 
         if (!c) return;	
@@ -250,6 +248,7 @@ static void cpqci_new_connection_mem_handle(CPQCICONTEXT Context, CPQCICONNECTIO
 	}
 }
 
+//We can assume that the connection lock is engaged
 static void cpqci_delete_connection(CPQCICONNECTIONEXTENSION* c) {
 
 	CPQCICONNECTIONEXTENSION** pc;
@@ -258,13 +257,10 @@ static void cpqci_delete_connection(CPQCICONNECTIONEXTENSION* c) {
 	pc = cpqci_find_connection(c);
 	if (!pc) return;
 
-	//Delete it out of the chain
-	down(&connection_list_lock);
 	if (*pc != NULL) {
 		DBG("Delete CONNECTIONOBJECT from chain\n");
 		*pc = c->Next;
 	}
-	up(&connection_list_lock);
 
 	cpqci_delete_connection_mem_handle(c);
 	DBG("(connection)kfree(%p)\n", c);
@@ -300,16 +296,23 @@ Return Value:
     	unsigned long long ref;
 	struct file* f = (struct file*) Context;
 
-	if (connectionExtension == NULL)
+	lock_kernel();
+	
+	if (connectionExtension == NULL) {
+		unlock_kernel();
 		return 0;
+	}
 
     	CpqCiOsGetRefCount(Context, (CPQCITYPEQUAD*) &ref);
-    	DBG(">>> Connection %d ref %ld\n", connectionExtension->Connection, ref);
+    	DBG(">>>pid %d: Connection %d ref %ld\n", current->pid, connectionExtension->Connection, ref);
 
 	if ( ref <= 4) {
 		//Right off the bat, delete the MemHandle
 		cpqci_delete_connection_mem_handle(connectionExtension);
 	}
+
+    	CpqCiOsGetRefCount(Context, (CPQCITYPEQUAD*) &ref);
+    	DBG(">>>pid %d: Connection %d ref %ld\n", current->pid, connectionExtension->Connection, ref);
 	
 	//  We only want to close a connection.
 	if (connectionExtension->Header.ObjectType == CpqCiConnection) {
@@ -334,18 +337,18 @@ Return Value:
 				else if (channel < CPQCI_BASE_CRITICALPRIORITY_CHANNEL)
 					    up(&adapterExtension->ChannelSem[1]);
 
-				DBG("*** Free Channel %d\n", channel);
+				DBG("pid: %d *** Free Channel %d\n", current->pid, channel);
 			}
 		
 			//This will close the connection extension object of the OS	
 			DBG("File %p private data gets deleted\n", f);
-			f->private_data = NULL;	
+			if (f) f->private_data = NULL;	
 			DBG("connection extension %p gets deleted\n", connectionExtension);
 			cpqci_delete_connection(connectionExtension);
 
-			DBG ("*** Free Connection %d\n", connection);
+			DBG("pid: %d *** Free Connection %d\n", current->pid, connection);
 		} else {
-			DBG (">>> Connection %d & Channel %d still in use.\n", connection, channel);
+			DBG ("pid: %d >>> Connection %d & Channel %d still in use.\n", current->pid, connection, channel);
 		}
 
 		if (f == NULL) {
@@ -358,6 +361,8 @@ Return Value:
 #ifndef DEBUG
 	//MOD_DEC_USE_COUNT;
 #endif
+	DBG(">>pid %d: close_helper finished\n", current->pid);
+	unlock_kernel();
 	return 0;
 }
 
@@ -418,7 +423,6 @@ void cleanup_module_helper(int irq_status, int irq, int register_status, struct
 	// Kill any open connection left
 	ci_conn = (adapterExtension == NULL) ? NULL : adapterExtension->ConnectionList; 
 	DBG("Delete any remaining open connections....\n");
-	//down(&connection_list_lock);
 	while (ci_conn != NULL) {
 		CPQCICONNECTIONEXTENSION* prev_ci_conn = ci_conn;
 		DBG("Deleting connection extension %p\n", prev_ci_conn);
@@ -427,7 +431,6 @@ void cleanup_module_helper(int irq_status, int irq, int register_status, struct
 		CpqCiShareContext(NULL, adapter, prev_ci_conn->Connection);
 		cpqci_close_helper(NULL, prev_ci_conn);
 	}
-	//up(&connection_list_lock);
 	DBG("Done");
 
 	//  Next we delete any adapter resources created.
@@ -486,7 +489,6 @@ Return Value:
 	connectionExtension = NULL;
 	error = CpqCiSuccess;
 	adapter = NULL;
-	sema_init(&connection_list_lock, 1);
 
 #ifdef CPQCI_X86_64
 	int ret;
@@ -497,6 +499,13 @@ Return Value:
 	ret |= register_ioctl32_conversion(CPQCI_IOCTL_OPEN_CHANNEL, cpqci_ioctl32_handler);
 	ret |= register_ioctl32_conversion(CPQCI_IOCTL_REGISTER_WAITER, cpqci_ioctl32_handler);
 	ret |= register_ioctl32_conversion(CPQCI_IOCTL_GET_INFO, cpqci_ioctl32_handler);
+
+//In the 2.4 timeframe NULL handlers were not allowed. 
+//We will sleep in the handler which results in locking problems, 
+//but this is only really a problem in 2.6. So we escaped doom by 
+//a hair.
+
+#if LINUX_VERSION_CODE < 0x020500
 	ret |= register_ioctl32_conversion(SEM_CREATE, cpqci_ioctl32_handler);
 	ret |= register_ioctl32_conversion(SEM_DESTROY, cpqci_ioctl32_handler);
 	ret |= register_ioctl32_conversion(SEM_UP, cpqci_ioctl32_handler);
@@ -504,6 +513,15 @@ Return Value:
 	ret |= register_ioctl32_conversion(SEM_DOWN_TIMER, cpqci_ioctl32_handler);
 	ret |= register_ioctl32_conversion(SEM_GET_VALUE, cpqci_ioctl32_handler);
 	ret |= register_ioctl32_conversion(SEM_SET_VALUE, cpqci_ioctl32_handler);
+#else
+	ret |= register_ioctl32_conversion(SEM_CREATE, NULL);
+	ret |= register_ioctl32_conversion(SEM_DESTROY, NULL);
+	ret |= register_ioctl32_conversion(SEM_UP, NULL);
+	ret |= register_ioctl32_conversion(SEM_DOWN, NULL);
+	ret |= register_ioctl32_conversion(SEM_DOWN_TIMER, NULL);
+	ret |= register_ioctl32_conversion(SEM_GET_VALUE, NULL);
+	ret |= register_ioctl32_conversion(SEM_SET_VALUE, NULL);
+#endif
 	if (ret)
 		printk(KERN_WARNING "cpqci:  Error registering ioctl32\n");
 #endif
@@ -575,6 +593,7 @@ Return Value:
 
 	sema_init(&adapterExtension->ChannelSem[0], 20);
 	sema_init(&adapterExtension->ChannelSem[1], 4);
+	sema_init(&connection_lock, 1);
 #if LINUX_VERSION_CODE >= 0x020300
 	for (channel = 0; channel < CPQCI_MAX_CHANNELS; channel++) {
 		init_waitqueue_head(&adapterExtension->RecvSignal[channel]);
@@ -724,8 +743,6 @@ int cpqci_ioctl32_handler(unsigned int fd, unsigned int cmd, unsigned long arg,
 	int err = 0;
 	int i = 0;
 	mm_segment_t old_fs;
-	sem_value_pair32 *data32 = (sem_value_pair32 *)arg;
-	sem_value_pair data64;
 	struct _CPQCIDRVOSMEMHANDLE32 *mh32 = (struct _CPQCIDRVOSMEMHANDLE32*)arg;
 	struct _CPQCIDRVOSMEMHANDLE mh64;
 	CPQCIDRVHANDLEPARMS32 *ub32 = (CPQCIDRVHANDLEPARMS*) arg;
@@ -769,6 +786,13 @@ int cpqci_ioctl32_handler(unsigned int fd, unsigned int cmd, unsigned long arg,
 		err = sys_ioctl(fd, cmd, (unsigned long)&ub64);
 		set_fs(old_fs);
 		break;
+	case SEM_CREATE: 
+	case SEM_DESTROY:
+	case SEM_UP:
+	case SEM_DOWN:
+	case SEM_DOWN_TIMER:
+	case SEM_GET_VALUE:
+	case SEM_SET_VALUE:
 	case CPQCI_IOCTL_REGISTER_WAITER:
 #ifdef IOCTL_GET_INFO
 	case CPQCI_IOCTL_GET_INFO:
@@ -777,22 +801,9 @@ int cpqci_ioctl32_handler(unsigned int fd, unsigned int cmd, unsigned long arg,
 		set_fs(KERNEL_DS);
 		err = sys_ioctl(fd, cmd, arg);
 		set_fs(old_fs);
-		break;
-	case SEM_CREATE:
-	case SEM_DESTROY:
-	case SEM_UP:
-	case SEM_DOWN:
-	case SEM_DOWN_TIMER:
-	case SEM_GET_VALUE:
-	case SEM_SET_VALUE:
-		err |= get_user(data64.sem, &data32->sem);
-		err |= get_user(data64.value, &data32->value);
-		err |= get_user(data64.rc, &data32->rc);
-		if (err) return -EFAULT;
-		old_fs = get_fs();
-		set_fs(KERNEL_DS);
-		err = sys_ioctl(fd, cmd, (unsigned long)&data64);
-		set_fs(old_fs);
+		
+
+break;
 	}
 	if(err == 0) {
 		switch(cmd) {
@@ -822,21 +833,18 @@ int cpqci_ioctl32_handler(unsigned int fd, unsigned int cmd, unsigned long arg,
 			err |= put_user(ub64.TimeOut, &ub32->TimeOut);
 			err |= put_user(ub64.Oshandle, &ub32->Oshandle);
 			break;
-		case CPQCI_IOCTL_REGISTER_WAITER:
-#ifdef IOCTL_GET_INFO
-		case CPQCI_IOCTL_GET_INFO:
-#endif
-			break;
-		case SEM_CREATE:
+		case SEM_CREATE: 
 		case SEM_DESTROY:
 		case SEM_UP:
 		case SEM_DOWN:
 		case SEM_DOWN_TIMER:
 		case SEM_GET_VALUE:
 		case SEM_SET_VALUE:
-			err |= put_user(data64.sem, &data32->sem);
-			err |= put_user(data64.value, &data32->value);
-			err |= put_user(data64.rc, &data32->rc);
+		case CPQCI_IOCTL_REGISTER_WAITER:
+#ifdef IOCTL_GET_INFO
+		case CPQCI_IOCTL_GET_INFO:
+#endif
+			break;
 		}
 	}
 	DBG("cpqci_ioctl32_handler exiting... err = %d\n", err) ;
@@ -887,26 +895,26 @@ Return Value:
 		c = (CPQCICONNECTIONEXTENSION*) f->private_data;
 		if (copy_from_user(&mh, (void*) arg, sizeof(mh))) {
 			status =  -EFAULT;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_0;
 		}
 		if (c == NULL) {
 			status =  -EFAULT;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_1;
 		}
 
-		down(&connection_list_lock);
+		lock_kernel();
 		cpqci_new_connection_mem_handle((CPQCICONTEXT)f, c, mh.MemorySize);
-		up(&connection_list_lock);
+		unlock_kernel();
 
 		if (c->MemHandleMapped == NULL) {
 			status =  -EFAULT;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_2;
 		}
 
 		DBG("copy_to_user(%p, %p, %x)\n", arg, c->MemHandleMapped, sizeof(struct _CPQCIDRVOSMEMHANDLE));
 		if (copy_to_user((void*) arg, c->MemHandleMapped, sizeof(mh))) {
 			status =  -EFAULT;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_2;
 		}
 		break;
 
@@ -914,20 +922,20 @@ Return Value:
 		c = (CPQCICONNECTIONEXTENSION*) f->private_data;
 		if (c == NULL) {
 			status =  -EFAULT;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_0;
 		}
 		if (copy_from_user(&mh, (void*) arg, sizeof(mh))) {
 			status =  -EFAULT;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_0;
 		}
 		//Sanity check on address passed in
 		if (mh.VirtualAddress != c->MemHandleMapped->VirtualAddress) {
 			status =  -EFAULT;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_1;
 		}
-		down(&connection_list_lock);
+		lock_kernel();
 		cpqci_delete_connection_mem_handle(c);
-		up(&connection_list_lock);
+		unlock_kernel();
 		break;
 
 	case CPQCI_IOCTL_CREATE_CHANNEL:
@@ -935,14 +943,16 @@ Return Value:
 		if (copy_from_user(ub, (CPQCIDRVHANDLEPARMS *) arg,
 				   sizeof(CPQCIDRVHANDLEPARMS))) {
 			status =  -EFAULT;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_0;
 		}
 		
+		lock_kernel();
 		c = cpqci_new_connection();
+		unlock_kernel();
 		DBG("New Connection Extension(create) %p\n", c);
 		if (c == NULL) {
 			status =-ENOMEM;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_2;
 		}
 
 		if (ub->Priority == CpqCiNormalPriority) {
@@ -962,7 +972,7 @@ Return Value:
 
 		if (status) {
 			status =-EBADF;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_2;
 		}
 
 		f->private_data = c;
@@ -1011,7 +1021,7 @@ Return Value:
 		} else {
 			ub->Error = CpqCiErrBadObjectType;
 			status = -EBADF;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_2;
 		}
 
 		if (ub->Error == CpqCiSuccess) {
@@ -1021,7 +1031,7 @@ Return Value:
 		} else {
 			DBG("CpqCiDrv!DriverDispatch: connection attempt failed.\n");
 			status = -1;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_2;
 		}
 		DBG("copy_to_user(%p, %p, %x)\n", arg, ub, sizeof(CPQCIDRVHANDLEPARMS));
 		copy_to_user((CPQCIDRVHANDLEPARMS *) arg, ub,
@@ -1039,14 +1049,16 @@ Return Value:
 		if (copy_from_user(ub, (CPQCIDRVHANDLEPARMS *) arg,
 				   sizeof(CPQCIDRVHANDLEPARMS))) {
 			status = -EFAULT;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_0;
 		}
 
+		lock_kernel();
 		c = cpqci_new_connection();
+		unlock_kernel();
 		DBG("New Connection Extension(open) %p\n", c);
 		if (c == NULL) {
 			status =-ENOMEM;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_2;
 		}
 
 		f->private_data = c;
@@ -1073,7 +1085,7 @@ Return Value:
 		} else {
 			ub->Error = CpqCiErrBadObjectType;
 			status = -EBADF;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_2;
 		}
 
 		if (ub->Error == CpqCiSuccess) {
@@ -1083,7 +1095,7 @@ Return Value:
 		} else {
 			DBG("CpqCiDrv!DriverDispatch: CpqCiDrvOpenChannel(CPQCISTATUS=%u).\n", ub->Error);
 			status = -1;
-			goto bad_ioctl_exit;
+			goto bad_ioctl_exit_2;
 		}
 		copy_to_user((CPQCIDRVHANDLEPARMS *) arg, ub,
 			     sizeof(CPQCIDRVHANDLEPARMS));
@@ -1155,19 +1167,22 @@ Return Value:
 	DBG("IOCTL cmd %x arg %p returns %d\n", cmd, arg, status);
 	return status;
 
-bad_ioctl_exit:
 
+bad_ioctl_exit_2:
+	if (c) { 
+		DBG("(connection)cpqci_delete_connection(%p)\n", c);
+		lock_kernel();
+		cpqci_delete_connection(c);
+		f->private_data = NULL; //c ??
+		unlock_kernel();
+	}
+bad_ioctl_exit_1:
 	if (arg && ub) {
 		copy_to_user((CPQCIDRVHANDLEPARMS *) arg, ub,
 		     sizeof(CPQCIDRVHANDLEPARMS));
 	}
 
-	if (c) { 
-		DBG("(connection)cpqci_delete_connection(%p)\n", c);
-		cpqci_delete_connection(c);
-		f->private_data = NULL; //c ??
-	}
-
+bad_ioctl_exit_0:
 	return status;
 }
 
@@ -1216,12 +1231,16 @@ int cpqci_mmap(struct file *f, struct vm_area_struct *vma)
 
 	if (nopage_method == 0) {
 		vma->vm_flags |= VM_IO;
+#if defined(MREMAP_PFN_RANGE)
+               	if (remap_pfn_range(vma, vma->vm_start, (pa)>>PAGE_SHIFT, size, vma->vm_page_prot))
+#else
 #if defined(MREMAP_PAGE_RANGE) || LINUX_VERSION_CODE >= 0x020600
                	if (remap_page_range(vma, vma->vm_start, pa, size, vma->vm_page_prot))
 		                        return -EAGAIN;
 #else
                	if (remap_page_range(vma->vm_start, pa, size, vma->vm_page_prot))
 	                        return -EAGAIN;
+#endif
 #endif
     		vma->vm_ops = &cpqci_vm_ops_high;
 	}
@@ -1844,7 +1863,7 @@ CpqCiOsGetRefCount(
 	if (!fp) return CpqCiSuccess;
 
 	RefCount->uint = atomic_read((atomic_t*)&fp->f_count);
-	DBG("RefCount: %p\n", RefCount->uint);
+	DBG("pid %d: RefCount %p\n", current->pid, RefCount->uint);
 
 	return CpqCiSuccess;
 }
