@@ -1,6 +1,6 @@
 /*
  ***************************************************************************
- * (C) Copyright Compaq Information Technologies Group, L.P. 2001, 2002    *
+ * (C) Copyright 2001-2005  Hewlett-Packard Development Company, L.P.      *
  ***************************************************************************
  *
  *  Title:    OS Utilities
@@ -249,7 +249,7 @@ void cleanup_module(void)
         if (gpCridInstanceData) gpCridInstanceData->ulStatus |= CARD_BUSY; 
 	//Unregister the driver at once, so that we have 
 	//clean break from agents.
-	unregister_chrdev(CRID_MAJOR, "cpqrid");
+	unregister_chrdev(crid_major, "cpqrid");
 #ifdef CRID_X86_64
 	unregister_ioctl32_conversion((('Z'<<8)|0));
 	unregister_ioctl32_conversion((('Z'<<8)|1));
