@@ -152,16 +152,16 @@ sub Intro {
    my $input ;
 
    print "-------------------------------------------------------------------------------
-   Welcome to the Setup Wizard for the HP System Management HomePage 2.0.0
+   Welcome to the Setup Wizard for the HP System Management HomePage 2.1.2
 
-   The Setup Wizard will install the HP System Management HomePage 2.0.0 on
+   The Setup Wizard will install the HP System Management HomePage 2.1.2 on
    your computer.
 
    On the following screens, you will configure security and access parameters
    for the HP System Management HomePage and related HP web-based management
    tools (e.g. Insight Management Agents, Version Control Agents, Diagnostics).
 
-   HP System Management HomePage 2.0.0
+   HP System Management HomePage 2.1.2
    Hewlett-Packard Company
    www.hp.com\n\n" ;
 
@@ -1336,7 +1336,8 @@ sub enterIPAddress {
    chomp ($IPAddress) ;
 
    if ($IPAddress ne "") {
-       $rc = &validIP ($IPAddress) ;
+#       $rc = &validIP ($IPAddress) ;
+       $rc = &validNetwork ($IPAddress) ;
    }
 
    return ($rc, $IPAddress) ;
